@@ -107,11 +107,14 @@ def parse_arguments():
     # export
     ap = subparsers.add_parser("export", help="export sequences")
     ap.set_defaults(func=export)
+    ap.add_argument("ALIASES",
+                    nargs="*",
+                    help="specific aliases to export")
     ap.add_argument("--instance-name", "-i", default=DEFAULT_INSTANCE_NAME_RO, help="instance name")
     ap.add_argument(
         "--namespace",
         "-n",
-        help="namespace name (e.g., RefSeq, NCBI, Ensembl, LRG)", )
+        help="namespace name (e.g., refseq, NCBI, Ensembl, LRG)", )
 
     # export aliases
     ap = subparsers.add_parser("export-aliases", help="export aliases")
@@ -120,7 +123,7 @@ def parse_arguments():
     ap.add_argument(
         "--namespace",
         "-n",
-        help="namespace name (e.g., RefSeq, NCBI, Ensembl, LRG)", )
+        help="namespace name (e.g., refseq, NCBI, Ensembl, LRG)", )
 
     # fetch-load
     ap = subparsers.add_parser("fetch-load", help="fetch remote sequences by accession and load them (low-throughput!)")
@@ -281,7 +284,22 @@ def export(opts):
     seqrepo_dir = os.path.join(opts.root_directory, opts.instance_name)
     sr = SeqRepo(seqrepo_dir)
 
-    if opts.namespace:
+    if opts.ALIASES:
+        def alias_generator():
+            for alias in set(opts.ALIASES):
+                yield from sr.aliases.find_aliases(namespace=opts.namespace,  # None okay
+                                                   alias=alias,
+                                                   translate_ncbi_namespace=True)
+        def _rec_iterator():
+            """yield (srec, [arec]) tuples to export"""
+            grouped_alias_iterator = itertools.groupby(alias_generator(),
+                                                       key=lambda arec: (arec["seq_id"]))
+            for seq_id, arecs in grouped_alias_iterator:
+                srec = sr.sequences.fetch_seqinfo(seq_id)
+                srec["seq"] = sr.sequences.fetch(seq_id)
+                yield srec, arecs
+        
+    elif opts.namespace:
         def _rec_iterator():
             """yield (srec, [arec]) tuples to export"""
             alias_iterator = sr.aliases.find_aliases(namespace=opts.namespace,
