@@ -1,8 +1,10 @@
+from collections.abc import Sequence
 import logging
 import os
 import re
 
 import bioutils.digests
+from bioutils.digests import seq_seqhash as sha512t24u
 
 from .seqaliasdb import SeqAliasDB
 from .fastadir import FastaDir
@@ -20,6 +22,58 @@ nsa_sep = u":"
 uri_re = re.compile(r"([^:]+):(.+)")
 
 
+
+class SequenceProxy(Sequence):
+    """Provides efficient and transparent string-like access, including
+    random access slicing and reversing, to a biological sequence that
+    is stored elsewhere.
+
+    """
+
+    def __init__(self, sr, namespace, alias):
+        self._sr = sr
+        self.seq_id = sr._get_unique_seqid(alias=alias, namespace=namespace)
+        self._md = sr.sequences.fetch_seqinfo(self.seq_id)
+
+    def _fetch(self, start=None, end=None):
+        seq_id = self.seq_id
+        return self._sr.sequences.fetch(seq_id, start, end)
+
+    def __bool__(self):
+        return str(self) != ""
+
+    def __eq__(self, s: str):
+        return str(self).__eq__(s)
+
+    def __getitem__(self, key):
+        if isinstance(key, int):
+            key = slice(key, key+1)
+        if key.step is not None:
+            raise ValueError("Only contiguous sequence slices are supported")
+        return self._fetch(key.start, key.stop)
+
+    def __hash__(self):
+        return hash(str(self))
+
+    def __len__(self):
+        return self._md["len"]
+
+    def __repr__(self):
+        return f"SequenceProxy(seq_id={self.seq_id}, len={len(self)})"
+
+    def __reversed__(self):
+        return self._fetch()[::-1]
+
+    def __str__(self):
+        return self._fetch()
+
+    @property
+    def aliases(self):
+        aliases = self._sr.aliases.find_aliases(seq_id=self.seq_id)
+        aliases = [":".join([a["namespace"], a["alias"]]) for a in aliases]
+        return aliases
+
+
 class SeqRepo(object):
     """Implements a filesystem-backed non-redundant repository of
     sequences and sequence aliases.
@@ -37,7 +91,7 @@ class SeqRepo(object):
 
     """
 
-    def __init__(self, root_dir, writeable=False, upcase=True, translate_ncbi_namespace=None, check_same_thread=False):
+    def __init__(self, root_dir, writeable=False, upcase=True, translate_ncbi_namespace=None, check_same_thread=False, use_sequenceproxy=True):
         self._root_dir = root_dir
         self._upcase = upcase
         self._db_path = os.path.join(self._root_dir, "aliases.sqlite3")
@@ -47,6 +101,7 @@ class SeqRepo(object):
         self._pending_aliases = 0
         self._writeable = writeable
         self._check_same_thread = True if writeable else check_same_thread
+        self.use_sequenceproxy = use_sequenceproxy
 
         if self._writeable:
             os.makedirs(self._root_dir, exist_ok=True)
@@ -67,9 +122,18 @@ class SeqRepo(object):
         return any(self.aliases.find_aliases(alias=a, namespace=ns))
 
     def __getitem__(self, nsa):
-        # lookup aliases, optionally namespaced, like NM_01234.5 or NCBI:NM_01234.5
+        """lookup aliases, optionally namespaced, like NM_01234.5 or NCBI:NM_01234.5
+
+        If SeqRepo was instantiated with use_sequenceproxy=True (the
+        default), a SequenceProxy is returned.
+
+        """
+
         ns, a = nsa.split(nsa_sep) if nsa_sep in nsa else (None, nsa)
-        return self.fetch(alias=a, namespace=ns)
+        if self.use_sequenceproxy:
+            return SequenceProxy(self, alias=a, namespace=ns)
+        else:
+            return self.fetch(alias=a, namespace=ns)
 
     def __iter__(self):
         """iterate over all sequences, yielding tuples of (sequence_record, [alias_records])
@@ -77,7 +141,7 @@ class SeqRepo(object):
         Both records are dicts.
         """
         for srec in self.sequences:
-            arecs = self.aliases.fetch_aliases(srec["seq_id"])
+            arecs = self.aliases.find_aliases(seq_id=srec["seq_id"])
             yield (srec, arecs)
 
     def __str__(self):
@@ -124,7 +188,7 @@ class SeqRepo(object):
             seq = seq.upper()
 
         try:
-            seqhash = bioutils.digests.seq_seqhash(seq)
+            seqhash = sha512t24u(seq)
         except Exception as e:
             import pprint
             _logger.critical("Exception raised for " + pprint.pformat(nsaliases))
@@ -154,7 +218,7 @@ class SeqRepo(object):
 
         # add/update external aliases for new and existing sequences
         # updating is optimized to load only new <seq_id,ns,alias> tuples
-        existing_aliases = self.aliases.fetch_aliases(seq_id)
+        existing_aliases = self.aliases.find_aliases(seq_id=seq_id)
         ea_tuples = [(r["seq_id"], r["namespace"], r["alias"]) for r in existing_aliases]
         new_tuples = [(seq_id, r["namespace"], r["alias"]) for r in nsaliases]
         upd_tuples = set(new_tuples) - set(ea_tuples)
@@ -181,9 +245,10 @@ class SeqRepo(object):
         if translate_ncbi_namespace is not None:
             _logger.warn("translate_ncbi_namespace is obsolete; translation is now automatic; this flag will be removed")
         seq_id = self._get_unique_seqid(alias=alias, namespace=namespace)
-        aliases = self.aliases.fetch_aliases(seq_id=seq_id)
+        aliases = self.aliases.find_aliases(seq_id=seq_id)
         if target_namespaces:
             aliases = [a for a in aliases if a["namespace"] in target_namespaces]
+        aliases = [nsa_sep.join([a["namespace"], a["alias"]]) for a in aliases]
         return aliases
 
 
@@ -196,10 +261,9 @@ class SeqRepo(object):
             _logger.warn("translate_ncbi_namespace is obsolete; translation is now automatic; this flag will be removed")
 
         namespace, alias = identifier.split(nsa_sep) if nsa_sep in identifier else (None, identifier)
-        aliases = self.translate_alias(alias=alias,
+        return self.translate_alias(alias=alias,
                                        namespace=namespace,
                                        target_namespaces=target_namespaces)
-        return [nsa_sep.join((a["namespace"], a["alias"])) for a in aliases]
 
 
     ############################################################################
